/**
 * @author Benjamin Hollis
 * 
 * Copyright 2007 Benjamin Hollis
 * See license.txt for additional license information.
 */

/**
 * @classDescription A class to hold information about individual Processes.
 * @constructor
 * @param {Number} processId
 * @param {String} name
 * @param {Number} kernelModeTime
 * @param {Number} userModeTime
 */
function ProcessInfo(processId, name, kernelModeTime, userModeTime) {
	this.processId = processId;
	this.name = name;
	this.kernelModeTime = Number(kernelModeTime);
	this.userModeTime = Number(userModeTime);
	this.totalTime = this.kernelModeTime + this.userModeTime;
}

function sortProcessesById(a,b) {
	return a.processId - b.processId;
}

/**
 * Sorts Processes by totalTime, with the largest total time first.
 */
function sortProcessesByTotalTime(a,b) {
	return b.totalTime - a.totalTime;
}

function getProcessStatistics() {
	try {
		var oWMI = GetObject("winmgmts://./root/cimv2");
		var cItems = oWMI.ExecQuery("Select * from Win32_Process");

		var processes = new Array();

		var cItem = new Enumerator(cItems);
		for (; !cItem.atEnd(); cItem.moveNext())
		{
			var processId = cItem.item().ProcessId;
			var name = cItem.item().Name;
			var kernelModeTime = cItem.item().KernelModeTime;
			var userModeTime = cItem.item().UserModeTime;
			
			processes.push(new ProcessInfo(processId, name, kernelModeTime, userModeTime));
		}
		
		processes.sort(sortProcessesById);
		
		return processes;
	}
	catch (err) {
		debug.innerHTML += "getProcessStatistics(): " + err.description + "<br>";
		throw err;
	}
}

function findCorrespondingProcess(process, oldProcesses, startIndex) {
	try {
		var index = startIndex;
		while(index < oldProcesses.length && process.processId != oldProcesses[index].processId) {
			index++;
		}
		
		if(index > oldProcesses.length) {
			//we didn't find a new process
			return { oldProcess: null, newIndex: startIndex };
		}
		else {
			return { oldProcess: oldProcesses[index], newIndex: index+1 };
		}
	}
	catch (err) {
		debug.innerHTML += "findCorrespondingProcess(): " + err.description + "<br>";
		throw err;
	}
}

function getTopProcesses(processes, oldProcesses, numTop) {
	try {
		var systemTotalTime = new Number(0);
		
		var topProcesses = new Array();
		
		var oldProcessIndex = 0;
		for(var i=0; i<processes.length; i++) {
			var process = processes[i];

			var oldInfo = findCorrespondingProcess(process, oldProcesses, oldProcessIndex);
			oldProcessIndex = oldInfo.newIndex;
			var oldProcess = oldInfo.oldProcess;
			
			if(oldProcess != null) {
				process.totalTime -= (oldProcess.kernelModeTime + oldProcess.userModeTime);
			}			
			
			systemTotalTime += process.totalTime;
			
			// ignore system processes
			if(process.processId != 0)
				topProcesses.push(process);
		}
		
		topProcesses.sort(sortProcessesByTotalTime);
		
		return { topProcesses: topProcesses.slice(0,numTop),
				 totalTime: systemTotalTime }; 
	}
	catch (err) {
		debug.innerHTML += "getTopProcesses(): " + err.description + "<br>";
		throw err;
	}
}

function fade(elementId, fade) {
	var element = document.getElementById(elementId);
	
	// clamp opacity to 0..1
	element.style.opacity = Math.min(1, Math.max(0, element.style.opacity + fade));
	element.style.filter = "alpha(opacity = " + element.style.opacity * 100 + ")"

	if(element.style.opacity === 0)
		element.parentNode.removeChild(element);
	else if(element.style.opacity === 1) {
		element.style.filter = '';

		/* there's got to be a better way to get rid of the ugly alpha effect */
		/*
		element.parentNode.removeChild(element);
		var newGadgetContent = document.createElement("div");
		newGadgetContent.id = element.id;
		newGadgetContent.className = "content";
		newGadgetContent.style.opacity = 1;
		
		newGadgetContent.innerHTML = element.innerHTML;
		
		document.body.appendChild(newGadgetContent);
		*/
		
		return;
	}
	else
		window.setTimeout("fade(\"" + element.id + "\"," + fade + ");", 10);
}

function updateGadgetContent(content) {
	try {
		if(!this.updateNum)
			this.updateNum = 0;
			
		this.updateNum++;
		
		var newGadgetContent = document.createElement("div");
		newGadgetContent.id = "gadgetContent" + this.updateNum;
		newGadgetContent.className = "content";
		
		newGadgetContent.innerHTML = content;
		newGadgetContent.style.opacity = 0;
		
		processList.appendChild(newGadgetContent);
		
		document.body.style.height = newGadgetContent.offsetHeight + 10 + 11;
		updateBackground(newGadgetContent.offsetHeight + 10 + 11);
		
		fade(newGadgetContent.id, .10);
		
		if(newGadgetContent.previousSibling && newGadgetContent.previousSibling.className === "content") {
			fade(newGadgetContent.previousSibling.id, -.10);
		}
	}
	catch (err) {
		debug.innerHTML += "updateGadgetContent(): " + err.description + "<br>";
		throw err;
	}
}

function updateBackground(height) {
	bg.style.height = height;
	window.middle.height = height - 11 - 10;
	
	if(window.middle.height % 2 > 0)
		window.middle.height += 1;
	
	//workaround for a bug:
	window.middle.top = 11 - (300-window.middle.height)/2;
	window.footer.top = height - 10;
}

function init() {
	LoadSettings();
	window.oldProcesses = getProcessStatistics();
	setTimeout("update()", 500);
	
	bg.style.height = document.body.style.height;
	bg.addImageObject("images/top.png",0,0);
	window.middle = bg.addImageObject("images/middle.png",0,11);
	window.footer = bg.addImageObject("images/bottom.png",0,55); 
}

function update() {
	try {
		var processes = getProcessStatistics();
		var topProcessInfo = getTopProcesses(processes, window.oldProcesses, window.numProcesses);			
		window.oldProcesses = processes;
		
		var topProcesses = topProcessInfo.topProcesses;
		var totalTime = topProcessInfo.totalTime;
		var result = "";
		for(var i = 0; i<topProcesses.length; i++) {
			var process = topProcesses[i];		
			var percentUsage = 	Math.round((process.totalTime / totalTime) * 10000) / 100
			result += "<tr" + (percentUsage > 50 ? " class=\"hotprocess\"" : "") + "><td class=\"processName\">" + process.name + "</td><td class=\"percentage\">" + percentUsage + "%<td>";
		}
		
		updateGadgetContent("<table>" + result + "</table>");
		
		setTimeout("update()", window.updateInterval);
	}
	catch (err) {
		debug.innerHTML += "update(): " + err.description + "<br>";
		throw err;
	}
}

/* Settings */
System.Gadget.settingsUI = "Settings.html";
System.Gadget.onSettingsClosed = LoadSettings;
function LoadSettings() {
	var numProcesses = System.Gadget.Settings.read("numProcesses");
	if (numProcesses != "")
		window.numProcesses = numProcesses;
	else
		window.numProcesses = 3;
		
	var updateInterval = System.Gadget.Settings.read("updateInterval");
	if (updateInterval != "")
		window.updateInterval = updateInterval * 1000;
	else
		window.updateInterval = 5000;
}